#!/usr/bin/perl

######################################################################
package Cognition::HTMLParser::Utils;
######################################################################

use CGI::Util;
use Cognition::MagicString;
use Roman;
use Text::Wrap;
use Time::HiRes qw(gettimeofday tv_interval); # TODO: add dep to website
use XML::LibXML qw(:all);
use strict;

our $BNodes    = {};

BEGIN
{
	our $VERSION   = 0.1;	
	our @outline   = ('outline:', 0, 0, 0, 0, 0, 0);
	our $lastLevel = 0;
	our $Timer     = undef;
}

sub searchClass
{
	my $target = shift;
	my $dom    = shift;
	
	return unless ($dom);
	
	my @matches = ();
	foreach my $node ($dom->getElementsByTagName('*'))
	{
		my $classList = $node->getAttribute('class');
		next unless (length $classList);
		
		if ($classList =~ / (^|\s) $target (\s|$) /x)
		{
			push @matches, $node;
		}
	}
	
	return @matches;
	
} #/sub searchClass


sub searchRel
{
	my $target = shift;
	my $dom    = shift;
	
	$target =~ s/[\:\.]/\[\:\.\]/;
	
	my @matches = ();
	for my $node ($dom->getElementsByTagName('*'))
	{
		my $classList = $node->getAttribute('rel');
		next unless (length $classList);
		
		if ($classList =~ / (^|\s) $target (\s|$) /ix)
		{
			push @matches, $node;
		}
	}
	
	return @matches;
	
} #/sub searchRel


sub searchID
{
	my $target = shift;
	my $dom    = shift;
	
	$target =~ s/^\#//;
	
	my @matches = ();
	for my $node ($dom->getElementsByTagName('*'))
	{
		my $id   = $node->getAttribute('id') || next;
		
		if ($id eq $target)
		{
			return $node;
		}
	}
	
} #/sub searchID


sub searchCXP
{
	my $target = shift;
	my $dom    = shift;
	
	$target =~ s/^\#//;
	
	my @matches = ();
	for my $node ($dom->getElementsByTagName('*'))
	{
		my $id   = $node->getAttribute('_xpath') || next;
		
		if ($id eq $target)
		{
			return $node;
		}
	}
	
} #/sub searchCXP


sub outline_point
{
	my $level = shift;
	$level =~ s/[^0-9]//g;
	
	if ($Cognition::HTMLParser::Utils::lastLevel < $level)
	{
		while ($Cognition::HTMLParser::Utils::lastLevel < $level)
		{
			$Cognition::HTMLParser::Utils::lastLevel++;
			$Cognition::HTMLParser::Utils::outline[$Cognition::HTMLParser::Utils::lastLevel] = 1;
		}
	}
	
	elsif ($Cognition::HTMLParser::Utils::lastLevel > $level)
	{
		while ($Cognition::HTMLParser::Utils::lastLevel > $level)
		{
			$Cognition::HTMLParser::Utils::outline[$Cognition::HTMLParser::Utils::lastLevel] = 1;
			$Cognition::HTMLParser::Utils::lastLevel--;
		}
		$Cognition::HTMLParser::Utils::outline[$level]++;
	}
	
	elsif ($Cognition::HTMLParser::Utils::lastLevel == $level)
	{
		$Cognition::HTMLParser::Utils::outline[$level]++;
	}
		
	my @rv = ();
	for (my $i=1; $i<=$level; $i++)
	{
		push @rv, $Cognition::HTMLParser::Utils::outline[$i];
	}
	
	return join('.', @rv);
}


sub XML_STRINGIFY
{
	my $dom        = shift;
	my $valueClass = shift || undef;

	# Clean up.
	my $clone = $dom->cloneNode(1);
	foreach my $kid ($clone->getElementsByTagName('*'))
	{
		$kid->removeAttribute('_xpath');
		$kid->removeAttribute('_xmllang');
		$kid->removeAttribute('microformats-content');
	}
	
	# Create string
	my $rv;
	foreach my $kid ($clone->childNodes)
	{
		$rv .= $kid->toString;
	}
	
	# Return string
	return (ms $rv, $dom);
}


sub STRINGIFY
{
	my $dom        = shift;
	my $valueClass = shift || undef;
	my $doABBR     = shift || (length $valueClass);
	my $noReduceWS = shift;
	my $joiner     = shift || '';
	my $str;

	return unless ($dom);

	# ABBR pattern
	if ($doABBR)
	{
		if ($dom->nodeType==XML_ELEMENT_NODE
			&& length $dom->getAttribute('microformats-content'))
		{
			my $title = $dom->getAttribute('microformats-content');
			return (ms $title, $dom);
		}
		elsif ( ($dom->nodeType==XML_ELEMENT_NODE 
			&& $dom->tagName eq 'abbr' 
			&& length $dom->getAttribute('title'))
		||   ($dom->nodeType==XML_ELEMENT_NODE 
			&& $dom->tagName eq 'acronym' 
			&& length $dom->getAttribute('title'))
		||   ($dom->nodeType==XML_ELEMENT_NODE
			&& $dom->getAttribute('title') =~ /data\:/)
		)
		{
			my $title = $dom->getAttribute('title');
	
			if ($title =~ / [\(\[] data\: (.*) [\)\]] /x
			||  $title =~ / data\: (.*) $ /x )
				{ $title = $1; }
	
			if (length $title)
				{ return (ms $title, $dom); }
		}
	}
	
	# Value excerpting.
	if (length $valueClass)
	{
		my @nodes = searchClass($valueClass, $dom);
		my @strs;
		if (@nodes)
		{
			foreach my $valueNode (@nodes)
			{
				push @strs, STRINGIFY($valueNode, undef, 1);
			}
			
			$str = ms (join $joiner, @strs), $dom;
		}
	}

	$str = _STRINGIFY($dom, 0, 0)
		unless (defined $str);

	unless ($noReduceWS)
	{
		# \x1D is used as a "soft" line break. It can be "absorbed" into an adjacent
		# "hard" line break.
		$str =~ s/\x1D+/\x1D/g;
		$str =~ s/\x1D\n/\n/gs;
		$str =~ s/\n\x1D/\n/gs;
		$str =~ s/\x1D/\n/gs;
		$str =~ s/(^\s+|\s+$)//gs;
	}
	
	return (ms $str, $dom);
}


sub _STRINGIFY
{
	my $domNode   = shift || return;
	my $inPRE     = shift || 0;
	my $indent    = shift || 0;
	my $rv = '';

	my $tag;
	if ($domNode->nodeType == XML_ELEMENT_NODE)
	{
		$tag = lc($domNode->tagName);
	}
	elsif ($domNode->nodeType == XML_COMMENT_NODE)
	{
		return (ms '');
	}
	
	# Change behaviour within <pre>.
	$inPRE++ if ($tag eq 'pre');
	
	# Text node, or equivalent.
	if (!$tag || $tag eq 'img' || $tag eq 'input')
	{
		$rv = $domNode->getData               if (!$tag);
		$rv = $domNode->getAttribute('alt')   if ($tag
		                                      && length $domNode->getAttribute('alt'));
		$rv = $domNode->getAttribute('value') if ($tag
		                                      && length $domNode->getAttribute('value'));

		unless ($inPRE)
		{
			$rv =~ s/[\s\r\n]+/ /gs;
		}
		
		return $rv;
	}
	
	# Breaks.
	return "\n" if ($tag eq 'br');
	return "\x1D\n====\n\n"
		if ($tag eq 'hr');
	
	# Deleted text.
	return '' if ($tag eq 'del');

	# Get stringified children.
	my (@parts, @ctags, @cdoms);
	my $extra = 0;
	if ($tag =~ /^([oud]l|blockquote)$/)
	{
		$extra += 6; # Advisory for word wrapping.
	}
	foreach my $child ($domNode->getChildNodes)
	{
		my $ctag = $child->nodeType==XML_ELEMENT_NODE ? lc($child->tagName) : undef;
		my $str  = _STRINGIFY($child, $inPRE, $indent + $extra);
		push @ctags, $ctag;
		push @parts, $str;
		push @cdoms, $child;
	}
	
	if ($tag eq 'ul' || $tag eq 'dir' || $tag eq 'menu')
	{
		$rv .= "\x1D";
		my $type = lc($domNode->getAttribute('type')) || 'disc';

		for (my $i=0; defined $parts[$i]; $i++)
		{
			next unless ($ctags[$i] eq 'li');
			
			$_ = $parts[$i];
			s/(^\x1D|\x1D$)//g;
			s/\x1D+/\x1D/g;
			s/\x1D\n/\n/gs;
			s/\n\x1D/\n/gs;
			s/\x1D/\n/gs;
			s/\n/\n    /gs;

			my $marker_type = $type;
			$marker_type = lc($cdoms[$i]->getAttribute('type'))
				if (length $cdoms[$i]->getAttribute('type'));

			my $marker = '*';
			if ($marker_type eq 'circle')    { $marker = '-'; }
			elsif ($marker_type eq 'square') { $marker = '+'; }
			
			$rv .= "  $marker $_\n";
		}
		$rv .= "\n";
	}
	
	elsif ($tag eq 'ol')
	{
		$rv .= "\x1D";
		
		my $count = 1;
		$count = $domNode->getAttribute('start')
			if (length $domNode->getAttribute('start'));
		my $type = $domNode->getAttribute('type') || '1';
		
		for (my $i=0; defined $parts[$i]; $i++)
		{
			next unless ($ctags[$i] eq 'li');
			
			$_ = $parts[$i];
			s/(^\x1D|\x1D$)//g;
			s/\x1D+/\x1D/g;
			s/\x1D\n/\n/gs;
			s/\n\x1D/\n/gs;
			s/\x1D/\n/gs;
			s/\n/\n    /gs;
			
			my $marker_value = $count;
			$marker_value = $cdoms[$i]->getAttribute('value')
				if (length $cdoms[$i]->getAttribute('value'));
			
			my $marker_type = $type;
			$marker_type = $cdoms[$i]->getAttribute('type')
				if (length $cdoms[$i]->getAttribute('type'));
				
			my $marker = sprintf('% 2d', $marker_value);
			if (uc($marker_type) eq 'A' && $marker_value > 0 && $marker_value <= 26)
				{ $marker = ' ' . chr( ord($marker_type) + $marker_value - 1 ); }
			elsif ($marker_type eq 'i' && $marker_value > 0 && $marker_value <= 3999)
				{ $marker = sprintf('% 2s', roman($marker_value)); }
			elsif ($marker_type eq 'I' && $marker_value > 0 && $marker_value <= 3999)
				{ $marker = sprintf('% 2s', Roman($marker_value)); }
				
			$rv .= sprintf("\%s. \%s\n", $marker, $_);

			$count++;
		}
		$rv .= "\n";
	}

	elsif ($tag eq 'dl')
	{
		$rv .= "\x1D";
		for (my $i=0; defined $parts[$i]; $i++)
		{
			next unless ($ctags[$i] eq 'dt' || $ctags[$i] eq 'dd');
			
			if ($ctags[$i] eq 'dt')
			{
				$rv .= $parts[$i] . ':';
				$rv =~ s/\:\s*\:$/\:/;
				$rv .= "\n";
			}
			elsif ($ctags[$i] eq 'dd')
			{
				$_ = $parts[$i];
				s/(^\x1D|\x1D$)//g;
				s/\x1D+/\x1D/g;
				s/\x1D\n/\n/gs;
				s/\n\x1D/\n/gs;
				s/\x1D/\n/gs;
				s/\n/\n    /gs;
				$rv .= sprintf("    \%s\n\n", $_);
			}
		}
	}

	elsif ($tag eq 'blockquote')
	{
		$rv .= "\x1D";
		for (my $i=0; defined $parts[$i]; $i++)
		{
			next unless ($ctags[$i]);
			
			$_ = $parts[$i];
			s/(^\x1D|\x1D$)//g;
			s/\x1D+/\x1D/g;
			s/\x1D\n/\n/gs;
			s/\n\x1D/\n/gs;
			s/\x1D/\n/gs;
			s/\n\n/\n/;
			s/\n/\n> /gs;
			$rv .= "> $_\n";
		}
		$rv =~ s/> $/\x1D/;
	}
	
	else
	{
		$rv = '';
		for (my $i=0; defined $parts[$i]; $i++)
		{
			$rv .= $parts[$i];
			
			# Hopefully this is a sensible algorithm for inserting whitespace
			# between childnodes. Needs a bit more testing though.
			
			# Don't insert whitespace if this tag or the next one is a block-level element.
			# Probably need to expand this list of block elements.
#			next if ($ctags[$i]   =~ /^(p|h[1-9]?|div|center|address|li|dd|dt|tr|caption|table)$/);
#			next if ($ctags[$i+1] =~ /^(p|h[1-9]?|div|center|address|li|dd|dt|tr|caption|table)$/);
			
			# Insert whitespace unless the string already ends in whitespace, or next
			# one begins with whitespace.
#			$rv .= ' '
#				unless ($rv =~ /\s$/ || (defined $parts[$i+1] && $parts[$i+1] =~ /^\s/));
		}
		
		if ($tag =~ /^(p|h[1-9]?|div|center|address|li|dd|dt|tr|caption|table)$/ && !$inPRE)
		{
			$rv =~ s/^[\t ]//s;
			local($Text::Wrap::columns);
			$Text::Wrap::columns = 78 - $indent;
			$rv = "\x1D".Text::Wrap::wrap('','',$rv);
			if ($tag =~ /^(p|h[1-9]?|address)$/)
			{
				$rv .= "\n\n";
			}
		}
		
		if ($tag eq 'sub')
			{ $rv = "($rv)"; }
		elsif ($tag eq 'sup')
			{ $rv = "[$rv]"; }
		elsif ($tag eq 'q')
			{ $rv = "\"$rv\""; }
		elsif ($tag eq 'th' || $tag eq 'td')
			{ $rv = "$rv\t"; }
	}

	return $rv;
}


sub xmlns
{
	my $ns  = shift;
	my $dom = shift;
	
	while ($dom->nodeType==XML_ELEMENT_NODE)
	{
		if (length $dom->getAttribute("xmlns:$ns"))
		{
			return $dom->getAttribute("xmlns:$ns");
		}
		$dom = $dom->getParentNode;
	}
	
	if ($dom->nodeType==XML_DOCUMENT_NODE)
	{
		if (length $dom->getDocumentElement->getAttribute("xmlns:$ns"))
		{
			return $dom->getDocumentElement->getAttribute("xmlns:$ns");
		}
	}
	
	return undef;
}

sub PrepareXPointers
# We save the node paths into special @_xpath attributes. Why? Because we often
# need to move DOM nodes about (e.g. uF include pattern) but still want access
# to the original node paths. Sometimes we're just dealing with document
# fragments, which have no meaningful node paths. This seems like an inefficient
# way of doing it, but I can't find a better solution.
{
	my $dom = shift;
	foreach my $tag ($dom->getElementsByTagName('*'))
		{ $tag->setAttribute('_xpath', $tag->nodePath); }
}

sub PrepareXmlLang
{
	my $dom  = shift;
	my $lang = shift || '';
	return unless ($dom->nodeType==XML_ELEMENT_NODE);
	
	if (length $dom->getAttribute('lang'))
		{ $lang = $dom->getAttribute('lang'); }
	elsif (length $dom->getAttribute('xml:lang'))
		{ $lang = $dom->getAttribute('xml:lang'); }
		
	$dom->setAttribute('_xmllang', $lang);
	
	foreach my $kid ($dom->getChildrenByTagName('*'))
	{
		PrepareXmlLang($kid, $lang);
	}
}

sub XPointer
{
	my $dom  = shift;
	my $page = shift || '';
	
	my $xpath = $dom->getAttribute('_xpath');
	if (!length $xpath)
		{ $xpath = $dom->nodePath; }
	
	my $escapedXPath = CGI::Util::escape($xpath);
	$escapedXPath =~ s|\%2f|/|ig;
	$escapedXPath =~ s|\%3f|?|ig;
	$escapedXPath =~ s|\%5b|[|ig;
	$escapedXPath =~ s|\%5d|]|ig;
	return "$page#xpointer($escapedXPath)";
}

sub BNode
{
	my $dom    = shift || undef;
	my $prefix = shift || 'Node';
	my $suffix = shift || 1;
	
	if ($dom)
	{
		my $xpath;
		if (ref $dom)
			{ $xpath = $dom->getAttribute('_xpath'); }
		else
			{ $xpath = $dom; }
		
		foreach my $key (keys %$Cognition::HTMLParser::Utils::BNodes)
		{
			if ($Cognition::HTMLParser::Utils::BNodes->{$key} eq $xpath
			&&  $key =~ /^ $prefix [0-9\.]+ $/x)
			{
				return $key;
			}
		}

		my $i = $suffix;
		while (defined $Cognition::HTMLParser::Utils::BNodes->{$prefix.$i}) { $i++; }
		$Cognition::HTMLParser::Utils::BNodes->{ $prefix.$i } = $xpath;
		return $prefix.$i;
	}
	
	my $i = 1;
	while (defined $Cognition::HTMLParser::Utils::BNodes->{$prefix.$i}) { $i++; }
	$Cognition::HTMLParser::Utils::BNodes->{ $prefix.$i } = 'anon';
	return $prefix.$i;
		
}

sub BNodeURI
{
	return 'bnode:///' . BNode(@_);
}


sub Debug
{
#	unless ($Cognition::HTMLParser::Utils::Timer)
#	{
#		$Cognition::HTMLParser::Utils::Timer = [gettimeofday];
#	}
#
#	my $elapsed = tv_interval $Cognition::HTMLParser::Utils::Timer;
#	my $message = shift;
#
#	print STDERR "$elapsed: $message\n";
}


1; #/package Cognition::HTMLParser::Utils




